/*
 * Copyright (c) 2007-2009 Chris Roberts <cpr420@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include <StorageKit.h>
#include <SupportKit.h>

#include <add-ons/tracker/TrackerAddOn.h>


// Choose a system apps directory based on the OS in use.
#if defined(__HAIKU__)
const directory_which kAppsDirectory = B_SYSTEM_APPS_DIRECTORY;
#else
const directory_which kAppsDirectory = B_BEOS_APPS_DIRECTORY;
#endif


void launch_terminal(BEntry *entry, const char* terminal) {

	BPath path;

	if (entry->GetPath(&path) != B_OK)
		return;


	// Change our working directory to the target folder.
	chdir(path.Path());


	// Launch the Terminal app.
	system (terminal);

}


void process_refs(entry_ref base_ref, BMessage *message, void *reserved) {

	// Store the working directory so we can reset it after we're done.
	char cwd[B_PATH_NAME_LENGTH + 1];

	//TODO Check the getcwd call for NULL??
	getcwd(cwd, B_PATH_NAME_LENGTH + 1);


	// Locate the system apps directory.
	BPath terminalPath;
	if (find_directory(kAppsDirectory, &terminalPath) != B_OK)
		return;


	// Add 'Terminal' to the path.
	if (terminalPath.Append("Terminal") != B_OK)
		return;


	//TODO Verify that the path actually points to the Terminal app.


	// The trailing '&' is so that the command forks to the background and doesn't
	// block tracker execution. (Only needed in Haiku??)
	BString terminalCommand(terminalPath.Path());
	terminalCommand << " &";


	BEntry entry;
	int32 index;
	entry_ref tracker_ref;

	// Iterate through the refs that Tracker has sent us.
	for (index = 0; message->FindRef("refs", index, &tracker_ref) == B_OK; index++) {

		// Pass 'true' as the traverse argument below, so that if the ref
		// is a symbolic link we get the target of the link to ensure that
		// it actually exists.
		if (entry.SetTo(&tracker_ref, true) == B_OK) {
			if (entry.IsDirectory())
				launch_terminal(&entry, terminalCommand.String());
		}

	}


	// If nothing was selected we'll use the base folder.
	if (index == 0) {
		if (entry.SetTo(&base_ref) == B_OK)
			launch_terminal(&entry, terminalCommand.String());
	}


	// Restore the original working directory.
	chdir(cwd);

}
